
//------------------------------------------------------------//
//                                                            //
//   Class for storage of company specific data               //
//                                                            //
//------------------------------------------------------------//

class CompanyData
{
	_company_id = null; // company id
	_goal_list = null; // index = GOLD, SILVER or BRONZE
	_transported_list = null; // index = 0, 1, 2 for the three cargoes (with lowest cargo ID first)
	_hq_sign = null; // sign id of the sign sitting ontop of the HQ

	_goal_ptr = null;

	constructor(company_id, goal_ptr)
	{
		this._company_id = company_id;
		this._goal_list = [-1, -1, -1];
		this._transported_list = [0, 0, 0];
		this._hq_sign = -1;

		this._goal_ptr = goal_ptr;
	}

	// Creates goal in the GUI
	function CreateGoals();
	function UpdateGoals();

	// Send information about goal to AI or Human players
	function InformAboutGoal();

	// Track goal progress
	function TrackProgress();

	// Called when the end of game has been reached
	function GameEnd();

	// Called at end of year (but not when the game end has been reached)
	function EndOfYear();

	// For save/load
	function SaveToTable();
	static function CreateFromTable(table, goal_ptr);
}

function CompanyData::CreateGoals()
{
	this.UpdateGoals();
}
function CompanyData::UpdateGoals()
{
	// Create goals for the goal GUI
	for(local i = 0; i < 3; i++)
	{
		local transport_target = GSController.GetSetting("transport_target");
		local should_exist = this._transported_list[i] < transport_target;
		local exists = GSGoal.IsValidGoal(this._goal_list[i]);

		Log.Info(GSCompany.GetName(this._company_id) + " - " + GSCargo.GetCargoLabel(this._goal_ptr._cargo_list[i]) + ":  target: " + transport_target + "  transported: " + this._transported_list[i] + "  exist: " + exists + "  should exist: " + should_exist, Log.LVL_DEBUG);

		// Exist, but should be removed?
		if(exists && !should_exist)
			GSGoal.Remove(this._goal_list[i]);

		// Doesn't exist, but should exist?
		if(!exists && should_exist)
		{
			this._goal_list[i] =
				GSGoal.New(this._company_id, GSText(GSText.STR_GOAL_GOLD + i, 
						// target
						transport_target,
						// goal cargoes
						1 << this._goal_ptr._cargo_list[0],
						1 << this._goal_ptr._cargo_list[1],
						1 << this._goal_ptr._cargo_list[2],
						// target year
						this._goal_ptr.GetEndYear()/*,
						// progress:
						this._transported_list[0],
						this._transported_list[1],
						this._transported_list[2]*/
					), GSGoal.GT_NONE, 0);
		}
	}
}

/* returns GOLD, SILVER, BRONZE or NO_MEDAL */
function CompanyData::GetMedal()
{
	local num_achived = 0;
	for(local i = 0; i < 3; i++)
	{
		if(this._transported_list[i] > GSController.GetSetting("transport_target"))
		{
			num_achived++;
		}
	}

	switch(num_achived)
	{
		case 3: return GOLD;
		case 2: return SILVER;
		case 1: return BRONZE;
		default: return NO_MEDAL;
	}
}

function CompanyData::InformAboutGoal()
{
	//if(g_scp.CanSpeakWith(this._company_id))
	//{
	//	g_scp.TellCompany("NewGoal", "TransportGoals", this._company_id, this._goal_value);
	//}

	// AIs can't read news, but humans can be on the same company as AIs,
	// so always send news even if the company has registered with the
	// SCP protocol.
	GSNews.Create(GSNews.NT_GENERAL, GSText(GSText.STR_NEW_GOAL_NEWS, 
			GSController.GetSetting("transport_target"),
			1 << this._goal_ptr._cargo_list[0],
			1 << this._goal_ptr._cargo_list[1],
			1 << this._goal_ptr._cargo_list[2]
		), this._company_id);
}

function CompanyData::TrackProgress()
{
	local town_list = GSTownList();
	local industry_list = GSIndustryList();

	for(local i = 0; i < 3; i++)
	{
		local cargo = this._goal_ptr._cargo_list[i];
		local transported = 0;

		// Scan all towns and industries for delivery of cargo since last check

		// Todo: do some optimization and only monitor towns/industries that can accept the given cargo
		foreach(town_id, _ in town_list)
		{
			transported += GSCargoMonitor.GetTownDeliveryAmount(this._company_id, cargo, town_id, true);
		}
		foreach(industry_id, _ in industry_list)
		{
			transported += GSCargoMonitor.GetIndustryDeliveryAmount(this._company_id, cargo, industry_id, true);
		}

		this._transported_list[i] += transported;
	}

	// Update goals
	this.UpdateGoals();
	this.UpdateHQSign();
}

function CompanyData::UpdateHQSign()
{
	// Todo, once basic functionality is done

/*	// Put the sign on the south tile of the HQ
	local hq_tile = Direction.GetAdjacentTileInDirection(
			GSCompany.GetCompanyHQ(this._company_id),
			Direction.DIR_S);

	local hq_exist = GSMap.IsValidTile(hq_tile);
	if(company_data._hq_sign != null && !hq_exist)
	{
		// Sign exist, but no HQ => remove sign
		GSSign.RemoveSign(company_data._hq_sign);
		company_data._hq_sign = null;
	}
	else if(hq_exist)
	{
		// The HQ exist
		local sign_text = GSText(GSText.STR_SCORE, (company_data._goal_value - GOAL_STEP_SIZE));
		if(company_data._hq_sign == null)
		{
			// HQ exist, but no sign yet => create new sign
			company_data._hq_sign = GSSign.BuildSign(hq_tile, sign_text);
		}
		else
		{
			// HQ exist as well as a sign
			if(GSSign.GetLocation(company_data._hq_sign) == hq_tile)
			{
				// The sign is at the right location => update only the text contents
				GSSign.SetName(company_data._hq_sign, sign_text);
			}
			else
			{
				// The sign exist, but at the wrong tile
				GSSign.RemoveSign(company_data._hq_sign);
				company_data._hq_sign = GSSign.BuildSign(hq_tile, sign_text);
			}
		}
	}
	else
	{
		// No HQ and no old sign
	}
*/
}

function CompanyData::GameEnd()
{
	GSGoal.Question(0, this._company_id, GSText(GSText.STR_GAME_END_GOLD + this.GetMedal(), 
			this._transported_list[0],
			1 << this._goal_ptr._cargo_list[0],
			this._transported_list[1],
			1 << this._goal_ptr._cargo_list[1],
			this._transported_list[2],
			1 << this._goal_ptr._cargo_list[2]
		), GSGoal.QT_INFORMATION, GSGoal.BUTTON_CLOSE);
}

function CompanyData::EndOfYear()
{
	GSNews.Create(GSNews.NT_GENERAL, GSText(GSText.STR_GOAL_PROGRESS_NEWS, 
			1 << this._goal_ptr._cargo_list[0],
			this._transported_list[0],
			1 << this._goal_ptr._cargo_list[1],
			this._transported_list[1],
			1 << this._goal_ptr._cargo_list[2],
			this._transported_list[2],
			GSController.GetSetting("transport_target")
		), this._company_id);
}

function CompanyData::SaveToTable()
{
	return {
		company_id = this._company_id,
		goal_list = this._goal_list,
		transported_list = this._transported_list,
		hq_sign = this._hq_sign,
	};
}

/* static */ function CompanyData::CreateFromTable(table, goal_ptr)
{
	local result = CompanyData(table.company_id, goal_ptr);

	result._goal_list = table.goal_list;
	result._transported_list = table.transported_list;
	result._hq_sign = table.hq_sign;

	return result;
}
