/*
 * This file is part of MinimalGS, which is a GameScript for OpenTTD
 * Copyright (C) 2012  Leif Linse
 *
 * MinimalGS is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License
 *
 * MinimalGS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MinimalGS; If not, see <http://www.gnu.org/licenses/> or
 * write to the Free Software Foundation, Inc., 51 Franklin Street, 
 * Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

/* Import SuperLib for GameScript */
import("util.superlib", "SuperLib", 25);
Result <- SuperLib.Result;
Log <- SuperLib.Log;
Helper <- SuperLib.Helper;
ScoreList <- SuperLib.ScoreList;
Tile <- SuperLib.Tile;
Direction <- SuperLib.Direction;
Town <- SuperLib.Town;
Industry <- SuperLib.Industry;

require("version.nut"); // get SELF_VERSION

require("goal.nut");
require("companydata.nut");


// Goals
GOLD <- 0;
SILVER <- 1;
BRONZE <- 2;

NO_MEDAL <- 3;


class MainClass extends GSController 
{
	_loaded_data = null;
	_loaded_from_version = null;

	_company_list = null;
	_goal_data = null;

	constructor()
	{
		this._loaded_data = null;
		this._loaded_from_version = null;

		this._company_list = [];
		this._goal_data = null; // created by Init
	}
}

function MainClass::Start()
{
	this.Init();

	Log.Info("Setup done", Log.LVL_INFO);

	// Wait for the game to start
	this.Sleep(1);

	local last_goal_check = GSDate.GetCurrentDate();
	local last_loop_year = GSDate.GetYear(GSDate.GetCurrentDate());
	while (true) {
		local loop_start_tick = GSController.GetTick();

		// Handle events
		this.HandleEvents();


		// Reached end of year?
		local current_date = GSDate.GetCurrentDate();
		local year = GSDate.GetYear(current_date);
		if(year != last_loop_year)
		{
			// reached end of challenge?
			if(year == this._goal_data.GetEndYear())
			{
				this.GameEnd();
			}
			else if(year < this._goal_data.GetEndYear())
			{
				this.EndOfYear();
			}

			last_loop_year = year;
		}
	
		// Once a month
		if(current_date > last_goal_check + 30)
		{
			last_goal_check = current_date;

			// Check goals setup once a month to fix problems when events has been lost
			this.UpdateCompanyList();

			// Has any goals been fulfilled?
			GSLog.Info("Scan goals START");
			this.ScanGoals();
			GSLog.Info("Scan goals END");
		}


		// Check for incoming SCP messages
		//Log.Info("SCP.Check", Log.LVL_INFO);
		//while(g_scp.Check()) {};

		// Loop with a frequency of five days
		local ticks_used = GSController.GetTick() - loop_start_tick;
		this.Sleep(Helper.Max(1, 5 * 74 - ticks_used));
	}
}

function MainClass::Init()
{
	if (this._loaded_data != null)
	{
		// Copy loaded data from this._loaded_data to this.*
		// or do whatever with the loaded data

		// load goal data
		this._goal_data = GoalData.CreateFromTable(this._loaded_data.goal_data);

		// Load company data from loaded data
		foreach(_, company_table in this._loaded_data.company_list)
		{
			if(company_table != null)
			{
				Log.Info("Loading data for company " + GSCompany.GetName(company_table.company_id), Log.LVL_INFO);
				this._company_list.append(
					CompanyData.CreateFromTable(company_table, this._goal_data)
				);
			}
		}

		this._loaded_data = null; // don't attempt to load again
	}
	else
	{
		// New game
		this._goal_data = GoalData();
		this._goal_data.InitalizeGoals(); // select goal cargoes
	}

	// Add possible new companies
	this.UpdateCompanyList();

	// Start up cargo monitoring by reading from
	// all cargo monitors.
	this.ScanGoals();
}

function MainClass::HandleEvents()
{
	if(GSEventController.IsEventWaiting())
	{
		local ev = GSEventController.GetNextEvent();

		if(ev == null)
			return;

		local ev_type = ev.GetEventType();
		if(ev_type == GSEvent.ET_COMPANY_NEW ||
				ev_type == GSEvent.ET_COMPANY_BANKRUPT ||
				ev_type == GSEvent.ET_COMPANY_MERGER)
		{
			Log.Info("A company was created/bankrupt/merged => update company list", Log.LVL_INFO);

			// Update the goal list when:
			// - a new company has been created
			// - a company has gone bankrupt
			// - a company has been bought by another company
			this.UpdateCompanyList();
		}
	}
}

function MainClass::UpdateCompanyList()
{
	// Loop over all possible company IDs
	for(local c = GSCompany.COMPANY_FIRST; c <= GSCompany.COMPANY_LAST; c++)
	{
		// has goals already been set up for this company?
		local existing = null;
		local existing_idx = 0;
		foreach(company_data in this._company_list)
		{
			if(company_data._company_id == c)
			{
				existing = company_data;
				break;
			}
			existing_idx++;
		}

		// does the company exist in the game
		if(GSCompany.ResolveCompanyID(c) == GSCompany.COMPANY_INVALID)
		{
			if(existing != null)
			{
				// Remove data for no longer existing company
				this._company_list.remove(existing_idx);
			}
			continue;
		}
	
		// If the company can be resolved and exist (goals has been setup) => don't do anything
		if(existing != null) continue;

		// Company goals has not yet been setup for this company
		local company_data = CompanyData(c, this._goal_data);
		company_data.CreateGoals();
		company_data.InformAboutGoal();
		company_data.TrackProgress();

		this._company_list.append(company_data);
	}
}

function MainClass::ScanGoals()
{
	foreach(company_data in this._company_list)
	{
		company_data.TrackProgress();
	}
}

// Called when the end of game has been reached
function MainClass::GameEnd()
{
	// Pause the game 
	GSGame.Pause();

	// Show End of Game message for each company
	foreach(company_data in this._company_list)
	{
		company_data.GameEnd();
	}

}

// Called at end of year (but not when the game end has been reached)
function MainClass::EndOfYear()
{
	foreach(company_data in this._company_list)
	{
		company_data.EndOfYear();
	}
}

function MainClass::Save()
{
	Log.Info("Saving data to savegame", Log.LVL_INFO);

	local company_save_list = [];
	foreach(company_data in this._company_list)
	{
		company_save_list.append(company_data.SaveToTable());
	}

	return { 
		goal_data = this._goal_data.SaveToTable(),
		company_list = company_save_list
	};
}

function MainClass::Load(version, tbl)
{
	Log.Info("Loading data from savegame made with version " + version + " of the game script", Log.LVL_INFO);

	if(version > SELF_VERSION)
	{
		Log.Warning("Warning: Loading from a newer version of TransportGoals", Log.LVL_INFO);
	}

	// Store a copy of the table from the save game
	// but do not process the loaded data yet. Wait with that to Init
	// so that OpenTTD doesn't kick us for taking too long to load.
	this._loaded_data = {}
   	foreach(key, val in tbl)
	{
		this._loaded_data.rawset(key, val);
	}	
	this._loaded_from_version = version;
}
